import asyncio
import pipes
import traceback

from . import client_utils, log


def async_cache(fxn):
    memo = {}

    async def wrapper(*args):
        key = tuple(args)
        if key not in memo:
            memo[key] = asyncio.ensure_future(fxn(*args))
        return await memo[key]

    return wrapper


async def run_and_check(*args, input=None, stop_on_error=True):
    log.info("Running:", " ".join(pipes.quote(a) for a in args))
    if input is not None:
        log.info("stdin:", input)
    PIPE = asyncio.subprocess.PIPE
    proc = await asyncio.subprocess.create_subprocess_exec(
        *args, stdin=PIPE, stdout=PIPE, stderr=PIPE
    )
    stdout, stderr = await proc.communicate(input=input)

    if stop_on_error and proc.returncode != 0:
        cmd_str = " ".join(pipes.quote(a) for a in args)
        try:
            raise client_utils.CalledProcessError(
                proc.returncode, cmd_str, stdout, stderr, "", stdin=input
            )
        except:
            traceback.print_exc()
            raise
    else:
        return stdout, stderr
